;; Generated package description from compact-docstrings.el  -*- no-byte-compile: t -*-
(define-package "compact-docstrings" "0.2" "Shrink blank lines in docstrings and doc comments" 'nil :commit "50b9ec928193f339c3c2ae20f088ea62cc82bb9d" :authors '(("Clément Pit-Claudel" . "clement.pitclaudel@live.com")) :maintainer '("Clément Pit-Claudel" . "clement.pitclaudel@live.com") :keywords '("convenience" "faces" "lisp" "maint" "c") :url "https://github.com/cpitclaudel/compact-docstrings")
